# CritLink Mediator: Dump package.
# Implements the M1dump class.  This token-stream source takes
# the place of an M1http object, sending a given string of data
# as the document instead of a fetching it over the network.
#
# Exports: ->dump($data, $url, $mimetype)

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
#
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
#
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
#
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

package M1dump;
use M1config;
use M1token;
use strict;

@M1dump::ISA = 'sprocket';             # inherit from "sprocket"

sub dump
    {
    my ($self,
        $data,                         # string: document data to send
        $url,                          # M1url: URL corresponding to the file
        $mimetype) = @_;               # string: MIME type for the document

    $self->send(new M1token('status:url', $url));
    $self->send(new M1token('response:200', 'OK'));
    $self->send(new M1token('header:content-type', $mimetype));

    my $datatype = 'data:binary';
    if ($mimetype =~ m#^text/#) { $datatype = 'data:text'; }

    $self->send(new M1token($datatype, $data));
    $self->send(new M1token('error:eof', ''));
    }
