# CritLink Mediator: Link object package.
# Implements the M1link class.  This object contains the structure
# representing a link (from an "source" to a "target").
#
# Exports: ->{'type'}, ->{'source'}, ->{'target'},
#          ->{'selected'}, ->{'ident'}, ->repr()

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

package M1link;
use M1url;
use repr;
use strict;

sub new
    {
    my $class = shift;
    my $self = {  'source' => shift,   # M1url: source document URL
                  'target' => shift,   # M1url: target document URL
                   'title' => shift,   # string: advisory title for target
                     'rel' => shift,   # string: role of target wrt source
                     'rev' => shift,   # string: role of source wrt target
                'selected' => shift,   # optional flag
                   'ident' => shift }; # optional unique identifier

    $self->{'type'} =~ tr/A-Z/a-z/;
    return bless($self, $class);
    }

sub repr                               # display the object (for &repr)
    {
    my ($self) = @_;
    my $result;

    $result .= 'sel ' if $self->{'selected'};
    $result .= ref($self->{'source'}) eq 'M1url' ?
        ($self->{'source'}->{'frag'} ? 'fine ' : 'coarse ') : 'none ';
    $result .= 'to ';
    $result .= ref($self->{'target'}) eq 'M1url' ?
        ($self->{'target'}->{'frag'} ? 'fine ' : 'coarse ') : 'none ';
    $result .= '[' . $self->{'ident'} . '] ' if $self->{'ident'};
    $result .= &repr::repr($self->{'source'});
    $result .= " as $self->{'rev'}" if $self->{'rev'};
    $result .= ' -> ';
    $result .= &repr::repr($self->{'target'});
    $result .= " ($self->{'title'})" if $self->{'title'};
    $result .= " as $self->{'rel'}" if $self->{'rel'};
    return $result;
    }

1;
