# CritLink Mediator: CGI output module.
# Implements the M1output class.  This sprocket is responsible for
# printing tokens to stdout as no-parse-headers CGI script output.
#
# Exports: ->recv($token)

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

package M1output;
use strict;
use M1med;

sub new
    {
    my ($class) = @_;
    return bless({}, $class);
    }

sub recv
    {
    my ($self, $token) = @_;

    if ($token->{'type'} =~ /^status:/)          # no printable output here
        {
        if ($' eq 'url') { $self->{'url'} = $token->{'data'}; }
        }

    elsif ($token->{'type'} =~ /^response:/)     # HTTP response line
        {
        print "HTTP/1.0 $' $token->{'data'} [fwd]\r\n";
        }

    elsif ($token->{'type'} =~ /^header:/)       # HTTP header
        {
        (my $header = $') =~ s/^./\U$&/;
        if ($header eq 'Server')
            {
            print "Server: $M1config::program\r\n";
            $self->{'serverdone'} = 1;
            }
        elsif ($header eq 'Location')
            { 
            my $url = $token->{'data'};
            if ($self->{'url'}->{'frag'})
                { $url .= '#' . $self->{'url'}->{'frag'}; }
            print "Location: " . &M1med::mediate($url) . "\r\n";
            }
        elsif ($header ne 'Content-length')
            { print "$header: $token->{'data'}\r\n"; }
        }

    elsif ($token->{'type'} !~ /^error:/)        # anything else
        {
        if (!$self->{'headersdone'})
            {
            if (!$self->{'serverdone'})
                { print "Server: $M1config::program\r\n"; }
            print "\r\n";
            $self->{'headersdone'} = 1;
            }
        $self->{'outbuf'} .= $token->canon();
        if (length($self->{'outbuf'}) > 4096)
            { $self->flush(); }
        }
    }

sub flush
    {
    my ($self) = @_;
    print $self->{'outbuf'};
    $self->{'outbuf'} = '';
    }

sub DESTROY
    {
    my ($self) = @_;
    $self->flush();
    }

1;
