# CritLink Mediator: Start/end/title finder package.
# Implements the M1start class.  This sprocket examines tokens from
# from the lexical analyzer and inserts a status token for the title,
# another just after the document body begins, and one before it ends.
#
# Exports: ->recv($token)

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

package M1start;
use strict;

@M1start::ISA = 'sprocket';            # inherit from "sprocket"

sub recv
    {
    my ($self, $token) = @_;
    my $type = $token->{'type'};

    if ($type eq 'data:text' or $type =~ /^tag:/) { $self->{'in-entity'} = 1; }
	if (!$self->{'in-entity'}) { $self->send($token); return; }

    # <TITLE>, <STYLE>, and <SCRIPT> elements may appear in the <HEAD>
    # element and contain textual content that is not part of the body.

    if ($type =~ /^tag:(\/?)(title|style|script)$/)
        { 
        $self->{"in-$2"} = ($1 ? 0 : 1);
        $self->send($token);
        if ($type eq 'tag:/title' and $self->{'title'})
            {
            $self->{'title'} =~ s/\s+/ /g;
            $self->{'title'} =~ s/^\s*|\s*$//g;
            $self->send(new M1token('status:title', $self->{'title'}));
            }
        return;
        }
    
    if ($type eq 'tag:/head' or $type eq 'tag:body')
        { $self->{'in-title'} = $self->{'in-style'} = 0; }

    if ($type eq 'data:text' and $self->{'in-title'})
        { $self->{'title'} .= $token->{'data'}; }
    
    if ($self->{'in-title'} or $self->{'in-style'} or $self->{'in-script'})
        { $self->send($token); return; }

    # </BODY>, </HTML>, or end-of-file signify the end of the body.

    if ($type eq 'tag:/body' or $type eq 'tag:/html' or $type eq 'error:eof')
        { $self->sendstart(); $self->sendend(); }

    # If we are already in the body, skip the extra work.

    if ($self->{'sent-start'})
        { $self->send($token); return; }

    # Textual content signifies the start of the body.

    if ($type eq 'data:text' and $token->{'data'} =~ /\S/)
        { $self->sendstart(); }

    # Any tags not allowed in the <HEAD> signify the start of the body.

    if ($type =~ /^tag:/ and
        $type !~ /^tag:(html|\/?head|body|link|meta|base|isindex|decl)$/)
        { $self->sendstart(); }

    $self->send($token);

    # If we got a <BODY> tag, the body begins immediately afterwards.

    if ($type eq 'tag:body')
        { $self->sendstart(); }
    }

sub sendstart
    {
    my ($self) = @_;
    if (!$self->{'sent-start'})
        {
        $self->send(new M1token('status:start'));
        $self->{'sent-start'} = 1;
        }
    }

sub sendend
    {
    my ($self) = @_;
    if (!$self->{'sent-end'})
        {
        $self->send(new M1token('status:end'));
        $self->{'sent-end'} = 1;
        }
    }

1;
