# CritLink Mediator: Terminal output package.
# Provides a few utility routines for sending coloured debugging
# output to a VT100-compatible terminal.
#
# Exports: ::msg($message), ::dbg($message), ::err($message)

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

package M1term;
use repr;
use M1config;

$old = select(STDERR);
$| = 1;
select($old);

use strict;

$M1term::error = "\x1b[31m";
$M1term::normal = "\x1b[0m";

%M1term::colour = ('M1config' => "\x1b[1;37m",
                   'M1token'  => "\x1b[36m",
                   'M1url'    => "\x1b[33m",

                   'M1gather' => "\x1b[1;35m",
                   'M1db'     => "\x1b[35m",

                   'M1http'   => "\x1b[34m",
                   'M1lex'    => "\x1b[1;34m",

                   'M1finder' => "\x1b[1;36m",
                   'M1ui'     => "\x1b[1;32m",
                   'M1med'    => "\x1b[1;33m");

sub msg
    {
    if ($M1config::showmessage)
        {
        my ($pkg) = (caller);
        my $msg = ref($_[0]) ? &repr::repr($_[0]) : $_[0];
        print STDERR $M1term::colour{$pkg} . $msg . $M1term::normal . "\n";
        }
    }

sub dbg
    {
    if ($M1config::showdebug)
        {
        my ($pkg) = (caller);
        my $msg = ref($_[0]) ? &repr::repr($_[0]) : $_[0];
        print STDERR $M1term::colour{$pkg} . $msg . $M1term::normal . "\n";
        }
    }

sub err
    {
    my $msg = ref($_[0]) ? &repr::repr($_[0]) : $_[0];
    print STDERR $M1term::error . $msg . $M1term::normal . "\n";
    }

1;
