# CritLink Mediator: Token package.
# Implements the M1token class.  This object represents a token
# and provides methods for display and for conversion to text.
#
# Exports: ->canon(), ->repr()

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

package M1token;
use M1term;
use repr;
use strict;

sub new
    {
    my ($class, $type, $data) = @_;
    return bless({'type' => $type,     # token type ('major:minor')
                  'data' => $data},    # any data structure
                 $class);
    }

sub canon
    {
    my ($type, $data) = @{$_[0]}{'type', 'data'};

    if ($type =~ /^tag:/)
        {
        my ($result, $key) = ($');

        if (!$data) { return "<$result>"; }
        if (!ref($data)) { return $data; }

        for $key (sort keys %$data)
            {
            if (exists($$data{$key}))
                {
                $result .= " $key";
                if ($$data{$key} ne '')
                    {
                    # If some evil JavaScript contains both kinds of quotes,
                    # we're screwed.  What genius thought this one up?
                    my $quote = ($$data{$key} =~ /"/) ? "'" : '"';
                    $result .= '=' . $quote . $$data{$key} . $quote;
                    }
                }
            }
        return "<$result>";
        }

    if ($type eq 'data:text')
        {
        $data =~ s/&/&amp;/g;
        $data =~ s/</&lt;/g;
        $data =~ s/>/&gt;/g;
        $data =~ s/[^\t\r\n\x20-\x7e]/'&#'.unpack('C',$&).';'/eg;
        return $data;
        }

    if ($type eq 'data:binary')
        { return $data; }

    return '';
    }

sub repr
    {
    my ($type, $data) = @{$_[0]}{'type', 'data'};
    if (!$data) { return $type; }

    my $data = ' ' . &repr::repr($data);
    if ($type =~ /^tag:/ and $data eq ' {}') { return $type; }
    return $type . $data;
    }

1;
