# Common Gateway Interface package.
# For general-purpose use by CGI scripts.

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

package cgi;
use M1config;
use M1url;

$method = $ENV{'REQUEST_METHOD'};
$query = $ENV{'QUERY_STRING'};
$contentlength = $ENV{'CONTENT_LENGTH'};
$contenttype = $ENV{'CONTENT_TYPE'};

$remoteaddr = $ENV{'REMOTE_ADDR'};
$remotehost = $ENV{'REMOTE_HOST'} || $ENV{'REMOTE_ADDR'};
$remoteport = $ENV{'REMOTE_PORT'};
$user = $ENV{'REMOTE_USER'};
$ident = $ENV{'REMOTE_IDENT'};

$serverhost = $ENV{'SERVER_NAME'};
$serverport = $ENV{'SERVER_PORT'};
if (!$serverport or $serverport == 80) { $server = $servername; }
else { $server = "$servername:$serverport"; }

$selector = $ENV{'SCRIPT_NAME'};
if ($pathinfo = $ENV{'PATH_INFO'})
    { $pathinfo =~ s/^\/*/\//; }

$requesturl = "http://$server$selector$pathinfo";
if ($method eq 'GET' and $query) { $requesturl .= "?$query"; }

sub readform
    {
    if ($method eq 'POST' && $contentlength)
        { sysread(STDIN, $query, $contentlength); }

    if ($query ne '')
        { &readquery($query); }
    }

sub readquery
    {
    my ($query) = @_;
    %cgi::form = ();
    foreach $pair (split(/[&;?]/, $query))
        {
        my ($name, $value) = split('=', $pair, 2);
        $name =~ tr/+/ /; $value =~ tr/+/ /;
        $cgi::form{&M1url::decode($name)} = &M1url::decode($value);
        }
    }

sub readcookie
    {
    foreach $chunk (split(';', $ENV{'HTTP_COOKIE'}))
        {
        next unless $chunk =~ /=/;
        my ($key, $value) = ($`, $');
        $key =~ s/^\s*|\s*$//g;
        $value =~ s/^\s*|\s*$//g;
        $cgi::cookie{&M1url::decode($key)} = &M1url::decode($value);
        }
    }

sub httpheaders
    {
    return if $httpheadersdone++;
    my ($response, @headers, $header) = @_;

    print "HTTP/1.0 $response\r\n";
    print "Server: $M1config::program\r\n";
    foreach $header (@headers)
        { print "$header\r\n"; }
    print "\r\n";
    }

sub log
    {
    my ($verb, $object, $user, $result, $logfile) = @_;
    open(LOG, '>>' . ($logfile || $M1config::logfile));
    ($s, $m, $h, $d, $l, $y) = localtime(time());
    $td = sprintf("%02d-%02d-%02d %02d:%02d:%02d", $y%100, $l, $d, $h, $m, $s);
    $result = " [$result]" if $result;
    printf LOG "$td $verb $object $user @ $cgi::remotehost$result\n";
    close(LOG);
    }

1;
