# Representation routines.
# Displays a printed representation of a structured Perl object.
# Participating objects need a "->repr()" method that returns a string.

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

package repr;
use strict;

sub repr
    {
    my ($object) = @_;

    if (! ref($object))
        {
        return 'undef' if !defined($object);
        return $& if $object =~ /^\d+$/;
        return &reprstr($object);
        }

    if (ref($object) eq 'SCALAR')
        {
        return '(' . &repr($$object) . ')';
        }

    if (ref($object) eq 'ARRAY')
        {
        my($item, $notfirst);
        my $result = '[';
        for $item (@$object)
            { $result .= ' ' if $notfirst++;
            $result .= &repr($item); }
        return $result . ']';
        }

    if (ref($object) eq 'HASH')
        {
        my($key, $notfirst);
        my $result = '{';
        for $key (sort keys %$object)
            { $result .= ' ' if $notfirst++;
            $result .= &repr($key) . '=>' . &repr($$object{$key}); }
        return $result . '}';
        }

    if (ref($object) eq 'REF')
        {
        return '<' . &repr($$object) . '>';
        }

    if (ref($object))
        {
        # eval() protection removed due to Perl 5.004 bug!
        my $repr = $object->repr();
        return '<' . ref($object) . ' ' . $repr . '>';
        }
    }

sub reprstr
    {
    my ($string) = @_;
    $string =~ s/\\/\\\\/g;
    $string =~ s/\r/\\r/g;
    $string =~ s/\n/\\n/g;
    $string =~ s/\t/\\t/g;
    $string =~ s/"/\\"/g;
    $string =~ s/[\x00-\x1f\x7f-\xff]/"\\x".unpack("H2",$&)/eg;
    return '"'.$string.'"';
    }

1;
