# Stream-processing object ("sprocket") package.
# The base class for other stream-processing objects.  All sprockets
# require a "->recv()" method which accepts an input token.  The
# "->send()" method is defined for convenience, and to establish the
# convention that the "next" member of a sprocket is a reference to
# the sprocket which receives its output.  The "->recv()" method
# defined here yields the identity sprocket.

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

package sprocket;
use strict;

sub new
    {
    my ($class, $next) = @_;
    return bless({'next' => $next},    # sprocket: the next sprocket
                 $class);
    }

sub recv
    {
    my ($self, $token) = @_;
    $self->send($token);               # pass on input to the next sprocket
    }

sub send
    {
    my ($self, $token) = @_;
    $self->{'next'}->recv($token);     # send a token to the next sprocket
    }

1;
