#!/usr/bin/perl
# CritLink Mediator: CGI interface for querying the link database.

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

# Look for "M1config.pm" in the lib/ sibling directory.
BEGIN { push(@INC, (($0 =~ /(.*)\//) ? $1 : '.') . '/../lib'); }

use M1config;

use cgi;
use M1db;
use M1link;
use M1url;

&cgi::readform;

($starturl = $cgi::pathinfo) =~ s/^\/*//;
if (!$starturl)
    {
    print "Content-Type: text/plain\n\nNo target URL specified.\n";
    exit;
    }
$start = new M1url($starturl);

print "Content-Type: text/linkinfo\n\n";

$db = new M1db($M1config::dbpath);
$getforward = sub { $db->fwdlinks($_[0]); };
$getbackward = sub { $db->backlinks($_[0]); };

if (defined($cgi::form{'forward'}))
    {
    $limit = ($cgi::form{'forward'} eq '') ? 1 : $cgi::form{'forward'};
    &scan($start, $limit, 'forwardlink', 'target', $getforward);
    }
elsif (defined($cgi::form{'backward'}))
    {
    $limit = ($cgi::form{'backward'} eq '') ? 1 : $cgi::form{'backward'};
    &scan($start, $limit, 'backwardlink', 'source', $getbackward);
    }
else
    { &scan($start, 0, 'forwardlink', 'target', $getforward); }

sub scan
    {
    my ($start, $limit, $dir, $member, $getlinks) = @_;
    my @here = ($start);
    my %seen;
    foreach $depth (reverse (0..$limit))
        {
        my @next = ();
        foreach $url (@here)
            {
            next if $seen{$url->canon('nofrag')};
            print $url->canon('nofrag') . "\n";
            $seen{$url->canon('nofrag')} = 1;
            my ($info) = $db->docinfo($url);
            foreach $key (keys %$info)
                { print "$key: $$info{$key}\n"; }
            if ($depth > 0)
                {
                my @links = &$getlinks($url);
                foreach $link (@links)
                    {
                    my $nextlink = $link->{$member};
                    push(@next, $nextlink);
                    my $source = $link->{'source'}->canon();
                    my $target = $link->{'target'}->canon();
                    print "$dir: $source $target ";
                    print "$link->{'rel'} $link->{'rev'}";
                    print "$link->{'title'}\n";
                    }
                }
            print "\n";
            }
        @here = @next;
        }
    }
