#!/usr/bin/perl
# CritWriter: Rapid publishing annotator.

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

# Look for "M1config.pm" in the lib/ sibling directory.
BEGIN { push(@INC, (($0 =~ /(.*)\//) ? $1 : '.') . '/../lib'); }

use M1config;

use cgi;
use sprocket;
use M1ui;
use M1db;
use M1url;

$ENV{'PATH'} = '';
$ENV{'SHELL'} = '/bin/sh';
$SIG{'ALRM'} = 'diealarm';
sub diealarm { exit; }
alarm $M1config::servertimeout;

my $DIVBAR_TEMPLATE = "$M1config::docpath/cw-divbar.html";
my $ANNOTATE_TEMPLATE = "$M1config::docpath/cw-annotate.html";
my $ERROR_TEMPLATE = "$M1config::docpath/cw-error.html";
my $ACCOUNT_TEMPLATE = "$M1config::docpath/cw-account.html";
my $PUBLIC_TEMPLATE = "$M1config::docpath/cw-public.html";

my $DOCUMENT_TEMPLATE = "$M1config::docpath/cw-document.html";
my $NOTIFY_TEMPLATE = "$M1config::docpath/cw-notify.msg";

my $db = new M1db($M1config::dbpath);

&M1config::setclient($ENV{'HTTP_USER_AGENT'});
&cgi::readform;

if (! $cgi::form{'action'})
    {
    $target = $cgi::form{'target'};
    &cgi::log('ANNOTATE', $target, $cgi::ident);

    if ($target !~ /\S/)
        { &finish('404 Not Found', 'No target',
                  'No target document has been selected to annotate.'); }

    $docinfo = $db->docinfo(new M1url($target));
    $targettitle = $$docinfo{'title'} || $target;
    &cgi::readcookie();
    $cookiename = $cgi::cookie{'Crit_name'};
    $cookieaddress = $cgi::cookie{'Crit_address'};
    $cookieusername = $cgi::cookie{'Crit_username'};
    $cookiepassword = $cgi::cookie{'Crit_password'};
    $targetanchor = $cgi::form{'targetanchor'};
    &finish('200 Annotation Form Follows', '', '', $ANNOTATE_TEMPLATE);
    }

sub reject { &finish('400 Invalid Input', 'Invalid input', @_); }

sub checkpath
    {
    local($path) = $_[0];
    $path =~ s#^\s*/*|\s*$##g;
    if (! $path) { $path = '[default filename]'; }
    local($titlestr) = $cgi::form{'title'};
    $titlestr =~ s/^\s+|\s+$//g;
    $titlestr =~ s/[^\w\s]//g;
    $titlestr =~ s/\s+/-/g;
    $path =~ s/\[default filename\]/$titlestr/;
    if (! $path)
        { &reject('There is no filename to publish the document to.'); }
    if ($path !~ /^[\/\w.-]*\w$/)
        { &reject('The given filename contained unacceptable characters.'); }
    return $path;
    }

sub randchar
    {
    my $alphabet = 'abdefghijkmnpqrtuyABDEFGHJKLMNPQRTUY23456789';
    my $choice = int(rand() * length($alphabet));
    return substr($alphabet, $choice, 1);
    }

$cgi::form{'target'} =~  s/^\s*|\s*$//g;
$cgi::form{'phrase'} =~  s/^\s*|\s*$//g;
$cgi::form{'title'} =~  s/^\s*|\s*$//g;
$cgi::form{'linktype'} =~  s/^\s*|\s*$//g;
$cgi::form{'name'} =~  s/^\s*|\s*$//g;
$cgi::form{'address'} =~  s/^\s*|\s*$//g;

if (!$cgi::form{'target'})
    { &reject('No target document was specified for the annotation.'); }
if (!$cgi::form{'title'})
    { &reject('No title was specified for the annotation.'); }
if (!$cgi::form{'body'})
    { &reject('No text was entered for the annotation.'); }
if (!$cgi::form{'linktype'})
    { &reject('No link relationship was chosen for the annotation.'); }
if (!$cgi::form{'name'})
    { &reject('No author name was entered for the annotation.'); }
if (!$cgi::form{'address'})
    { &reject('No e-mail address was entered for the annotation author.'); }
if ($cgi::form{'address'} =~ /^([\w\.=-]+)\@([\w\.=-]+\.[a-z]+)$/i)
    { $addruser = $1; $addrdomain = $2; $address = "$addruser\@$addrdomain"; }
else
    { &reject("The specified e-mail address doesn't appear to be valid."); }
if ($cgi::form{'action'} =~ /account/i && $M1config::userdir)
    { $path = &checkpath($cgi::form{'homepath'}) . ".html"; }
if ($cgi::form{'action'} =~ /public/i && $M1config::pubpath)
    {
    $path = &checkpath($cgi::form{'pubpath'}) . ".html";

    $url = $M1config::puburl;
    $url =~ s#/*$#/#;
    $url .= "$addrdomain/$addruser/$path";
    $file = $M1config::pubpath;
    $file =~ s#/*$#/#;
    $file .= "$addrdomain/$addruser/$path";

    if (-e $file)
        {
        &finish('400 Invalid input', 'File exists',
                "A file at <a href=\"$url\">$url</a> already exists.
                 Please go back and try a different location.");
        }

    if (open(PASSWORD, "$M1config::pwpath/$addrdomain/$addruser"))
        { $password = <PASSWORD>; close(PASSWORD); }
    else
        {
        srand(time() ^ (unpack("%16C*", $address)<<16) ^ $$);
        foreach $i (1..8) { $password .= &randchar(); }
        system("/bin/mkdir -p $M1config::pwpath/$addrdomain");
        open(PASSWORD, ">$M1config::pwpath/$addrdomain/$addruser");
        print PASSWORD $password;
        close(PASSWORD);
        }
    }

if ($cgi::form{'coarse'} =~ /on/)      # no problem: no fragid necessary
    {
    }
elsif ($cgi::form{'targetanchor'})     # fragid is the anchor name
    {
    $fragment = "#" . $cgi::form{'targetanchor'};
    }
else                                   # build fragid from phrase words
    {
    if (!($phrase = $cgi::form{'phrase'}))
        { &reject('No target phrase was specified for the annotation.'); }
    $context = $cgi::form{'context'};
    $phrase =~ s/[^A-Za-z0-9']+/ /g;
    $phrase =~ s/^\s*|\s*$//g;
    $phrase =~ s/'//g;
    $phrase =~ s/\s+/-/g;
    if ($phrase !~ /\w/)
        { &reject("The target phrase contains no letters or digits."); }
    $context =~ s/[^A-Za-z0-9']+/ /g;
    $context = $phrase if $context !~ /\S/;
    $context =~ s/^\s*|\s*$//g;
    $context =~ s/'//g;
    $context =~ s/\s+/-/g;
    if ($context !~ /-.*-.*-/ and length($context) < 20)
        { &reject('The context phrase is probably too short to ' .
                  'ensure a unique match when finding the target.'); }
    if ($context !~ $phrase)
        { &reject("The context phrase doesn't contain the target phrase."); }

    $fragment = "#:words:" . $` . '(' . $& . ')' . $';
    }

$notebody = $cgi::form{'body'};
$notebody =~ s/\r\n/\n/g;
if ($cgi::form{'notation'} eq 'rapid')
    {
    $notebody =~ s/&/&amp;/g;                              # escaping
    $notebody =~ s/</&lt;/g;
    $notebody =~ s/>/&gt;/g;
    $notebody =~ s#\*\*(\w+)\*\*#<strong>$1</strong>#g;    # **bold**
    $notebody =~ s#\*(\w+)\*#<em>$1</em>#g;                # *emphasis*
    $notebody = "\n$notebody\n";
    $notebody =~ s#\n(([ \t]+\S.*\n)+)#\n<pre>$1</pre>#g;  # preformatted text
    $notebody =~ s#\n(\s*\n)+#\n<p>#g;                     # paragraph breaks
                                                           # hyperlinks
    $notebody =~ s#(http|ftp)://[-/\w\#().,:~?;'%]*[/\w]#<a href="$&">$&</a>#g;
    $notebody =~ s#([\w\.=-]+)\@([\w\.=-]+\.[a-z]+)#<a href="mailto:$&">$&</a>#g;
    $notebody =~ s/^([ \t]*\n)+//;
    $notebody =~ s/([ \t]*\n)+$//;
                                                           # bulleted lists
    $notebody =~ s%(<p>\s*\+\s+([^<]|<[^p])*)+% do
        { my $list = $&; $list =~ s/<p>\s*\+/<li>/g; "<ul>$list</ul>"; } %eg;
    }

$notetitle = $cgi::form{'title'};
$linktype = $cgi::form{'linktype'};
$author = $cgi::form{'name'};
$linktarget = $cgi::form{'target'} . $fragment;
$document = &M1ui::render($DOCUMENT_TEMPLATE, 'M1config');
$cookielife = $cgi::form{'cookielife'} * 3600 * 24;
if ($cgi::form{'cookiesave'} =~ /on/i)
    { $cookielife = 30 * 3600 * 24; }

sub cookieheader
    {
    my ($key, $value, $life) = @_;
    my ($s, $m, $h, $d, $l, $y, $w) = gmtime(time() + $life);
    my (@wn) = split(' ', 'Sun Mon Tue Wed Thu Fri Sat');
    my (@ln) = split(' ', 'Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec');
    my ($expiry) = sprintf("%3s, %02d-%3s-%04d %02d:%02d:%02d GMT",
    	$wn[$w], $d, $ln[$l], $y+1900, $h, $m, $s);
    return "Set-Cookie: Crit_$key=" . &M1url::encode($value) .
        "; expires=$expiry; path=/";
    }

if ($cgi::form{'monitor'} =~ /on/i)
    {
    my $urlob = new M1url($url);
    &M1term::msg("turning on monitoring");
    $docinfo = $db->docinfo($urlob);
    $$docinfo{'monitor'} = $cgi::form{'address'};
    $db->store($urlob);
    &M1term::msg("stored");
    }

if ($cgi::form{'action'} =~ /account/i)
    {
    use ftpput;

    local($url) = "http://$M1config::server/~$cgi::form{'username'}/$path";
    my $server = $M1config::server;
    $server =~ s/:\d+//;
    my $error = &ftpput::store($server,
                               $cgi::form{'username'},
                               $cgi::form{'password'},
                               "$M1config::userdir/$path",
                               $document);

    &cgi::log('ACCOUNT', $url, $address, $error);

    if ($error)
        {
        &finish('500 File Transfer Failed', 'File transfer failed',
            "Sorry, we were unable to place your file on the server. $error");
        }
    else
        {
	&finish('200 Comment Published', '', '', $ACCOUNT_TEMPLATE,
            &cookieheader('name', $cgi::form{'name'}, $cookielife),
            &cookieheader('address', $cgi::form{'address'}, $cookielife),
            &cookieheader('username', $cgi::form{'username'}, $cookielife),
            &cookieheader('password', $cgi::form{'password'}, $cookielife));
        }
    }
elsif ($cgi::form{'action'} =~ /public/i)
    {
    my $dir = $file;
    $dir =~ s#/[^/]*$##;

    &cgi::log('PUBLIC', $url, $address);
    &cgi::log('PUBLIC', $url, $address, undef,
              "$M1config::datapath/public.log");

    system("/bin/mkdir -p $dir");
    if (!open(FILE, "> $file"))
        {
        &finish('500 Server Error', 'Could not publish comment', <<MESSAGE);
Sorry!  Your comment could not be published
because of an error: <em>$!</em>.
MESSAGE
        }
    print FILE $document;
    close FILE;
    chmod 0644, $file;

    if ($M1config::notify)
        {
        if (-x $M1config::mailbin)
            {
            open(MAIL, '|' . $M1config::sendmail);
            print MAIL &M1ui::render($NOTIFY_TEMPLATE, 'M1config');
            close(MAIL);
            }
        }



    if (fork())
        {
        &finish('200 Comment Published', '', '', $PUBLIC_TEMPLATE,
            &cookieheader('name', $cgi::form{'name'}, $cookielife),
            &cookieheader('address', $cgi::form{'address'}, $cookielife));
	}

    use M1dump;
    use M1lex;
    use M1start;
    use M1gather;

    new M1dump(
        new M1lex(
            new M1start(
                new M1gather($db)
                )
            )
        )->dump($document, new M1url($url), 'text/html');
    exit;
    }
elsif ($cgi::form{'action'} =~ /download/i)
    {
    &cgi::log('DOWNLOAD', $target, $cgi::form{address});

    my $file = ($cgi::form{'url'} =~ /.*\/(.+)/) ? $1 : 'note.html';
    &cgi::httpheaders('200 OK',
        "Content-Type: application/octet-stream; name=\"$file\"",
        "Content-Disposition: inline; filename=\"$file\"",
        "Content-Length: " . length($document));
    print $document;
    exit;
    }

sub finish
    {
    local ($response, $title, $message, $template, @headers) = @_;
    &cgi::httpheaders($response, 'Content-Type: text/html', @headers);
    print &M1ui::render($template || $ERROR_TEMPLATE, 'M1config', 'cgi');
    exit;
    }
