#!/usr/bin/perl
# CritWriter: Annotation maintainer and editor.

# The CritLink Mediator and CritWriter software is copyright
# 1997 by Ka-Ping Yee (http://www.lfw.org/ping/).  All rights
# reserved.  Written for a project of the Foresight Institute.
# 
# This software is provided by Ka-Ping Yee under the following
# license.  By using, copying, or modifying this software, you
# agree that you understand and will comply with the following
# terms and conditions: permission to use, copy, modify, and
# distribute this software and documentation for any purpose
# is granted, PROVIDED (a) that the full text of this notice
# appears on all copies of the software and documentation or
# portions or derivations thereof, and (b) that the preceding
# paragraph, containing the copyright, is displayed (together
# with other copyright notices if any) by all products that
# incorporate this software or portions or derivations thereof.
# If possible, we prefer that the URL be an active hyperlink.
# 
# THIS SOFTWARE IS PROVIDED "AS IS".  KA-PING YEE AND FORESIGHT
# INSTITUTE MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, REPRESENTATIONS OR
# WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
# PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY
# THIRD-PARTY PATENTS, COPYRIGHTS, TRADEMARKS, OR OTHER RIGHTS.
# NEITHER KA-PING YEE NOR FORESIGHT INSTITUTE WILL BEAR ANY
# LIABILITY FOR ANY USE OF THIS SOFTWARE OR DOCUMENTATION.
# 
# The name and trademarks of Ka-Ping Yee and the Foresight
# Institute may not be used in publicity about this software
# without prior written permission (please contact us at
# <ping@foresight.org>).  Title to copyright in this software
# and documentation will at all times remain with the author.

# Look for "M1config.pm" in the lib/ sibling directory.
BEGIN { push(@INC, (($0 =~ /(.*)\//) ? $1 : '.') . '/../lib'); }

use M1config;

use cgi;
use M1ui;
use M1db;
use M1url;

$ENV{'PATH'} = '';
$ENV{'SHELL'} = '/bin/sh';
$SIG{'ALRM'} = 'diealarm';
sub diealarm { exit; }
alarm $M1config::servertimeout;

$LOGIN_TEMPLATE = "$M1config::docpath/cw-login.html";
$BADLOGIN_TEMPLATE = "$M1config::docpath/cw-badlogin.html";
$ERROR_TEMPLATE = "$M1config::docpath/cw-error.html";
$RMOK_TEMPLATE = "$M1config::docpath/cw-rmok.html";
$RMFAIL_TEMPLATE = "$M1config::docpath/cw-rmfail.html";
$RMCONF_TEMPLATE = "$M1config::docpath/cw-rmconf.html";
$REMINDOK_TEMPLATE = "$M1config::docpath/cw-remindok.html";

$REMIND_TEMPLATE = "$M1config::docpath/cw-remind.msg";

($target = $cgi::pathinfo) =~ s#^/+##;
&cgi::readform;
$target = $cgi::form{'target'} || $target;
$target =~ s#http:/+#http://#;                   # workaround for MSIE 4 bug
                                                 # which replaces // with /

if ($target =~ $M1config::puburlmatcher)
    {
    $addrdomain = $1;
    $addruser = $2;
    $address = "$addruser\@$addrdomain";
    $path = $3;
    }
else
    { &reject("No editable document was specified."); }

if (!(-f "$M1config::pubpath/$addrdomain/$addruser/$path"))
    { &reject("The specified document $target does not exist."); }

$db = new M1db($M1config::dbpath);
$docinfo = $db->docinfo(new M1url($target));
$targettitle = $$docinfo{'title'} || $target;
$name = $$docinfo{'author'} || 'no name';

if (open(PASSWORD, "$M1config::pwpath/$addrdomain/$addruser"))
    {
    $password = <PASSWORD>;
    close(PASSWORD);

    $cgi::form{'password'} =~ s/^\s*|\s*$//g;
    $password =~ s/^\s*|\s*$//g;
    }
else
    { &reject('This user has never registered a password.'); }

if ($cgi::form{'action'} =~ /remind/i)
    {
    &cgi::log('REMIND', $password, $address);
    open(FOO, '>/tmp/foo');
    print FOO &M1ui::render($REMIND_TEMPLATE, 'M1config');
    close(FOO);

    if (-x $M1config::mailbin)
        {
        open(MAIL, '|' . $M1config::sendmail);
        print MAIL &M1ui::render($REMIND_TEMPLATE, 'M1config');
        close(MAIL);
        }

    &finish('200 Reminder Sent', '', '', $REMINDOK_TEMPLATE);
    }

if (! $cgi::form{'password'})
    {
    &cgi::log('LOGIN', $target, $address);
    &finish('200 Editing Login Dialog Follows', '', '', $LOGIN_TEMPLATE);
    }

if ($password ne $cgi::form{'password'})
    {
    &cgi::log('BADPASSWORD', $target, $address);
    &finish('400 Password Incorrect', '', '', $BADLOGIN_TEMPLATE);
    }

$cgi::form{'target'} =~  s/^\s*|\s*$//g;

if ($cgi::form{'action'} =~ /remove/i)
    {
    if ($cgi::form{'action'} =~ /confirm/i)
        {
        $targeturl = new M1url($target);
        $db = new M1db($M1config::dbpath);
        $db->store($targeturl);

        &cgi::log('REMOVE', $target, $address);
        if (unlink("$M1config::pubpath/$addrdomain/$addruser/$path"))
            { &finish('200 Document Removed', '', '', $RMOK_TEMPLATE); }
        else
            { &finish('400 Error', '', '', $RMFAIL_TEMPLATE); }
        }
    else
        {
        &cgi::log('CONFIRM', $target, $address);
        &finish('200 Please Confirm', '', '', $RMCONF_TEMPLATE);
        }
    }
elsif ($cgi::form{'action'} =~ /edit/i)
    {
    &cgi::log('EDIT', $target, $address);
    &reject("Sorry, document editing is not yet available.");
    }

sub finish
    {
    local ($response, $title, $message, $template, @headers) = @_;
    &cgi::httpheaders($response, 'Content-Type: text/html', @headers);
    print &M1ui::render($template || $ERROR_TEMPLATE, 'M1config', 'cgi');
    exit;
    }

sub reject { &finish('400 Invalid Input', 'Invalid input', @_); }
