#!/usr/bin/perl

# Look for "M1config.pm" in the lib/ sibling directory.
BEGIN { push(@INC, (($0 =~ /(.*)\//) ? $1 : '.') . '/../lib'); }

use M1config;
use M1db;
use M1url;

$history = 50;

open(TAIL, "/usr/bin/tail -$history $M1config::datapath/public.log |");
while (<TAIL>)
    {
    ($date, $time, $event, $url, $address) = split(' ');
    next unless $event eq 'PUBLIC';
    $recent{$url} = 1;
    }
close(TAIL);

foreach $url (keys %recent)
    {
    $file = $url;
    $file =~ s/$M1config::puburl/$M1config::pubpath/;
    $mtime = (stat($file))[9];
    push(@recent, "$mtime $url $file") if $mtime;
    }
$count = scalar(@recent);

print <<TOP;
Content-Type: text/html

<!doctype html public "-//W3C//DTD HTML 3.2//EN">
<html><head>
<meta name=author content="an automated script">
<meta name=trigger content=never>
<meta name=notify content="send: !*">
<title>Recent Annotations</title>
</head><body bgcolor="#ffffff">
<h1>Recent Annotations</h1>

<p>The following are the $count most recently published or
edited annotations in the public area on this server.

<p><table>
<tr><th>date<th>title<th>size
TOP

$db = new M1db($M1config::dbpath);
foreach $entry (reverse sort @recent)
    {
    my ($mtime, $url, $file) = split(' ', $entry);
    my ($s, $m, $h, $d, $l, $y, $w) = localtime($mtime);
    my $datetime = sprintf('%2d:%02d:%02d %3s %2d %3s',
        $h, $m, $s, (qw/Sun Mon Tue Wed Thu Fri Sat/)[$w], $d,
        (qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/)[$l]);

    $file =~ s#^\./##;
    my $info = $db->docinfo(new M1url($url));
    my $author = $info->{'author'} || $info->{'authorlink'};
    $author = '<a href="' . $info->{'authorlink'} . '">' . $author . '</a>'
        if $info->{'authorlink'} ne '';
    my $title = $info->{'title'} || $file;

    $datetime =~ s/ /&nbsp;/g;
    $size =~ s/ /&nbsp;/g;
    print "<tr valign=top><td align=right><small>$datetime</small>\n";
    print "    <td><a href=\"$url\">$title</a>\n";
    print "    <small>by $author</small>\n" if $author ne '';
    print "    <td align=right>" . (-s $file) . "\n";
    }
print "</table></body></html>\n";
